@@ -3,7 +3,7 @@
 /**
  *
  *
- * @version $Id: Config.class.php 13036 2009-10-12 21:59:48Z lem9 $
+ * @version $Id$
  * @package phpMyAdmin
  */
 
@@ -92,7 +92,7 @@ class PMA_Config
      */
     function checkSystem()
     {
-        $this->set('PMA_VERSION', '3.2.2.1');
+        $this->set('PMA_VERSION', '3.3.8.1');
         /**
          * @deprecated
          */
@@ -489,7 +489,7 @@ class PMA_Config
     }
 
     /**
-     * verifies the permissions on config file (if asked by configuration) 
+     * verifies the permissions on config file (if asked by configuration)
      * (must be called after config.inc.php has been merged)
      */
     function checkPermissions()
@@ -554,7 +554,7 @@ class PMA_Config
      */
     function getThemeUniqueValue()
     {
-        return intval((null !== $_SESSION['PMA_Config']->get('fontsize') ? $_SESSION['PMA_Config']->get('fontsize') : (isset($_COOKIE['pma_fontsize']) ? $_COOKIE['pma_fontsize'] : 0))) + ($this->source_mtime + $this->default_source_mtime + $_SESSION['PMA_Theme']->mtime_info + $_SESSION['PMA_Theme']->filesize_info) . (isset($_SESSION['userconf']['custom_color']) ? substr($_SESSION['userconf']['custom_color'],1,6) : '');
+        return intval((null !== $_SESSION['PMA_Config']->get('fontsize') ? $_SESSION['PMA_Config']->get('fontsize') : (isset($_COOKIE['pma_fontsize']) ? $_COOKIE['pma_fontsize'] : 0))) + ($this->source_mtime + $this->default_source_mtime + $_SESSION['PMA_Theme']->mtime_info + $_SESSION['PMA_Theme']->filesize_info) . (isset($_SESSION['tmp_user_values']['custom_color']) ? substr($_SESSION['tmp_user_values']['custom_color'],1,6) : '');
     }
 
     /**
@@ -662,6 +662,7 @@ class PMA_Config
             // Backslashes returned by Windows have to be changed.
             // Only replace backslashes by forward slashes if on Windows,
             // as the backslash could be valid on a non-Windows system.
+            $this->checkWebServerOs();
             if ($this->get('PMA_IS_WINDOWS') == 1) {
                 $path = str_replace("\\", "/", dirname($url['path'] . 'a'));
             } else {
@@ -676,6 +677,11 @@ class PMA_Config
                     $path = dirname(dirname($path));
                 }
             }
+
+            // PHP's dirname function would have returned a dot when $path contains no slash
+            if ($path == '.') {
+                $path = '';
+            }
             // in vhost situations, there could be already an ending slash
             if (substr($path, -1) != '/') {
                 $path .= '/';
