@@ -3,8 +3,12 @@
 /**
  * Set of functions used to build SQL dumps of tables
  *
- * @version $Id: sql.php 11190 2008-04-06 23:22:19Z lem9 $
+ * @package phpMyAdmin-Export-SQL
+ * @version $Id$
  */
+if (! defined('PHPMYADMIN')) {
+    exit;
+}
 
 /**
  *
@@ -23,6 +27,7 @@ if (isset($plugin_list)) {
             'mime_type' => 'text/x-sql',
             'options' => array(
                 array('type' => 'text', 'name' => 'header_comment', 'text' => 'strAddHeaderComment'),
+                array('type' => 'bool', 'name' => 'include_comments', 'text' => 'strComments'),
                 array('type' => 'bool', 'name' => 'use_transaction', 'text' => 'strEncloseInTransaction'),
                 array('type' => 'bool', 'name' => 'disable_fk', 'text' => 'strDisableForeignChecks'),
                 ),
@@ -54,15 +59,16 @@ if (isset($plugin_list)) {
             $plugin_list['sql']['options'][] =
                 array('type' => 'bgroup', 'name' => 'structure', 'text' => 'strStructure', 'force' => 'data');
             if ($plugin_param['export_type'] == 'table') {
-                if (PMA_Table::_isView($GLOBALS['db'], $GLOBALS['table'])) {
+                if (PMA_Table::isView($GLOBALS['db'], $GLOBALS['table'])) {
                     $drop_clause = 'DROP VIEW';
                 } else {
                     $drop_clause = 'DROP TABLE';
                 }
-            } elseif (PMA_MYSQL_INT_VERSION >= 50000) {
-                $drop_clause = 'DROP TABLE / VIEW / PROCEDURE / FUNCTION';
             } else {
-                $drop_clause = 'DROP TABLE';
+		$drop_clause = 'DROP TABLE / VIEW / PROCEDURE / FUNCTION';
+		if (PMA_MYSQL_INT_VERSION > 50100) {
+                    $drop_clause .= ' / EVENT';
+                }
             }
             $plugin_list['sql']['options'][] =
                 array('type' => 'bool', 'name' => 'drop_table', 'text' => sprintf($GLOBALS['strAddClause'], $drop_clause));
@@ -73,7 +79,7 @@ if (isset($plugin_list)) {
             $plugin_list['sql']['options'][] =
                 array('type' => 'bool', 'name' => 'backquotes', 'text' => 'strUseBackquotes');
             $plugin_list['sql']['options'][] =
-                array('type' => 'bool', 'name' => 'procedure_function', 'text' => sprintf($GLOBALS['strAddClause'], 'CREATE PROCEDURE / FUNCTION'));
+                array('type' => 'bool', 'name' => 'procedure_function', 'text' => sprintf($GLOBALS['strAddClause'], 'CREATE PROCEDURE / FUNCTION' . (PMA_MYSQL_INT_VERSION > 50100 ? ' / EVENT' : '')));
 
             /* MIME stuff etc. */
             $plugin_list['sql']['options'][] =
@@ -84,10 +90,6 @@ if (isset($plugin_list)) {
                 $plugin_list['sql']['options'][] =
                     array('type' => 'bool', 'name' => 'relation', 'text' => 'strRelations');
             }
-            if (!empty($GLOBALS['cfgRelation']['commwork']) && PMA_MYSQL_INT_VERSION < 40100) {
-                $plugin_list['sql']['options'][] =
-                    array('type' => 'bool', 'name' => 'comments', 'text' => 'strComments');
-            }
             if (!empty($GLOBALS['cfgRelation']['mimework'])) {
                 $plugin_list['sql']['options'][] =
                     array('type' => 'bool', 'name' => 'mime', 'text' => 'strMIME_MIMEtype');
@@ -103,9 +105,9 @@ if (isset($plugin_list)) {
         $plugin_list['sql']['options'][] =
             array('type' => 'bgroup', 'name' => 'data', 'text' => 'strData', 'force' => 'structure');
         $plugin_list['sql']['options'][] =
-            array('type' => 'bool', 'name' => 'columns', 'text' => 'strCompleteInserts');
+            array('type' => 'bool', 'name' => 'columns', 'text' => 'strCompleteInserts', 'doc' => array('programs', 'mysqldump', 'option_mysqldump_complete-insert-option'));
         $plugin_list['sql']['options'][] =
-            array('type' => 'bool', 'name' => 'extended', 'text' => 'strExtendedInserts');
+            array('type' => 'bool', 'name' => 'extended', 'text' => 'strExtendedInserts', 'doc' => array('programs', 'mysqldump', 'option_mysqldump_extended-insert-option'));
         $plugin_list['sql']['options'][] =
             array('type' => 'text', 'name' => 'max_query_size', 'text' => 'strMaximalQueryLength');
         $plugin_list['sql']['options'][] =
@@ -115,7 +117,7 @@ if (isset($plugin_list)) {
         $plugin_list['sql']['options'][] =
             array('type' => 'bool', 'name' => 'hex_for_blob', 'text' => 'strHexForBLOB');
         $plugin_list['sql']['options'][] =
-            array('type' => 'select', 'name' => 'type', 'text' => 'strSQLExportType', 'values' => array('INSERT', 'UPDATE', 'REPLACE'));
+            array('type' => 'select', 'name' => 'type', 'text' => 'strSQLExportType', 'values' => array('INSERT' => 'INSERT', 'UPDATE' => 'UPDATE', 'REPLACE' => 'REPLACE'));
         $plugin_list['sql']['options'][] =
             array('type' => 'egroup');
     }
@@ -129,16 +131,35 @@ if (! isset($sql_backquotes)) {
 }
 
 /**
- * Outputs comment
+ * Possibly outputs comment
  *
  * @param   string      Text of comment
  *
- * @return  string      The formatted comment 
+ * @return  string      The formatted comment
  */
 function PMA_exportComment($text = '')
 {
-    // see http://dev.mysql.com/doc/refman/5.0/en/ansi-diff-comments.html
-    return '--' . (empty($text) ? '' : ' ') . $text . $GLOBALS['crlf'];
+    if (isset($GLOBALS['sql_include_comments']) && $GLOBALS['sql_include_comments']) {
+        // see http://dev.mysql.com/doc/refman/5.0/en/ansi-diff-comments.html
+        return '--' . (empty($text) ? '' : ' ') . $text . $GLOBALS['crlf'];
+    } else {
+        return '';
+    }
+}
+
+/**
+ * Possibly outputs CRLF
+ *
+ * @return  string  $crlf or nothing 
+ */
+function PMA_possibleCRLF()
+{
+
+    if (isset($GLOBALS['sql_include_comments']) && $GLOBALS['sql_include_comments']) {
+        return $GLOBALS['crlf'];
+    } else {
+        return '';
+    }
 }
 
 /**
@@ -156,11 +177,11 @@ function PMA_exportFooter()
     $foot = '';
 
     if (isset($GLOBALS['sql_disable_fk'])) {
-        $foot .=  $crlf . 'SET FOREIGN_KEY_CHECKS=1;' . $crlf;
+        $foot .=  'SET FOREIGN_KEY_CHECKS=1;' . $crlf;
     }
 
     if (isset($GLOBALS['sql_use_transaction'])) {
-        $foot .=  $crlf . 'COMMIT;' . $crlf;
+        $foot .=  'COMMIT;' . $crlf;
     }
 
     // restore connection settings
@@ -189,8 +210,13 @@ function PMA_exportHeader()
     global $cfg;
     global $mysql_charset_map;
 
-    if (PMA_MYSQL_INT_VERSION >= 40100 && isset($GLOBALS['sql_compatibility']) && $GLOBALS['sql_compatibility'] != 'NONE') {
-        PMA_DBI_try_query('SET SQL_MODE="' . $GLOBALS['sql_compatibility'] . '"');
+    if (isset($GLOBALS['sql_compatibility'])) {
+        $tmp_compat = $GLOBALS['sql_compatibility'];
+        if ($tmp_compat == 'NONE') {
+            $tmp_compat = '';
+        }
+        PMA_DBI_try_query('SET SQL_MODE="' . $tmp_compat . '"');
+        unset($tmp_compat);
     }
     $head  =  PMA_exportComment('phpMyAdmin SQL Dump')
            .  PMA_exportComment('version ' . PMA_VERSION)
@@ -200,7 +226,8 @@ function PMA_exportHeader()
     $head .=  PMA_exportComment($GLOBALS['strGenTime']
            . ': ' .  PMA_localisedDate())
            .  PMA_exportComment($GLOBALS['strServerVersion'] . ': ' . substr(PMA_MYSQL_INT_VERSION, 0, 1) . '.' . (int) substr(PMA_MYSQL_INT_VERSION, 1, 2) . '.' . (int) substr(PMA_MYSQL_INT_VERSION, 3))
-           .  PMA_exportComment($GLOBALS['strPHPVersion'] . ': ' . phpversion());
+           .  PMA_exportComment($GLOBALS['strPHPVersion'] . ': ' . phpversion())
+           .  PMA_possibleCRLF();
 
     if (isset($GLOBALS['sql_header_comment']) && !empty($GLOBALS['sql_header_comment'])) {
         // '\n' is not a newline (like "\n" would be), it's the characters
@@ -209,38 +236,38 @@ function PMA_exportHeader()
         $head .= PMA_exportComment();
         foreach($lines as $one_line) {
             $head .= PMA_exportComment($one_line);
-        } 
+        }
         $head .= PMA_exportComment();
     }
 
     if (isset($GLOBALS['sql_disable_fk'])) {
-        $head .=  $crlf . 'SET FOREIGN_KEY_CHECKS=0;' . $crlf;
+        $head .=  'SET FOREIGN_KEY_CHECKS=0;' . $crlf;
     }
 
     /* We want exported AUTO_INCREMENT fields to have still same value, do this only for recent MySQL exports */
     if (!isset($GLOBALS['sql_compatibility']) || $GLOBALS['sql_compatibility'] == 'NONE') {
-        $head .=  $crlf . (PMA_MYSQL_INT_VERSION >= 40101 ? 'SET SQL_MODE="NO_AUTO_VALUE_ON_ZERO";' . $crlf : ''); 
+        $head .=  'SET SQL_MODE="NO_AUTO_VALUE_ON_ZERO";' . $crlf;
     }
 
     if (isset($GLOBALS['sql_use_transaction'])) {
-        $head .=  $crlf .'SET AUTOCOMMIT=0;' . $crlf
+        $head .=  'SET AUTOCOMMIT=0;' . $crlf
                 . 'START TRANSACTION;' . $crlf;
     }
 
-    $head .= $crlf;
+    $head .= PMA_possibleCRLF();
 
     if (! empty($GLOBALS['asfile'])) {
         // we are saving as file, therefore we provide charset information
         // so that a utility like the mysql client can interpret
-        // the file correctly 
+        // the file correctly
         if (isset($GLOBALS['charset_of_file']) && isset($mysql_charset_map[$GLOBALS['charset_of_file']])) {
-            // $cfg['AllowAnywhereRecoding'] was true so we got a charset from 
+            // $cfg['AllowAnywhereRecoding'] was true so we got a charset from
             // the export dialog
             $set_names = $mysql_charset_map[$GLOBALS['charset_of_file']];
         } else {
             // by default we use the connection charset
-            $set_names = $mysql_charset_map[$GLOBALS['charset']]; 
-        } 
+            $set_names = $mysql_charset_map[$GLOBALS['charset']];
+        }
         $head .=  $crlf
                . '/*!40101 SET @OLD_CHARACTER_SET_CLIENT=@@CHARACTER_SET_CLIENT */;' . $crlf
                . '/*!40101 SET @OLD_CHARACTER_SET_RESULTS=@@CHARACTER_SET_RESULTS */;' . $crlf
@@ -269,22 +296,71 @@ function PMA_exportDBCreate($db)
         }
     }
     $create_query = 'CREATE DATABASE ' . (isset($GLOBALS['sql_backquotes']) ? PMA_backquote($db) : $db);
-    if (PMA_MYSQL_INT_VERSION >= 40101) {
-        $collation = PMA_getDbCollation($db);
-        if (strpos($collation, '_')) {
-            $create_query .= ' DEFAULT CHARACTER SET ' . substr($collation, 0, strpos($collation, '_')) . ' COLLATE ' . $collation;
-        } else {
-            $create_query .= ' DEFAULT CHARACTER SET ' . $collation;
-        }
+    $collation = PMA_getDbCollation($db);
+    if (strpos($collation, '_')) {
+        $create_query .= ' DEFAULT CHARACTER SET ' . substr($collation, 0, strpos($collation, '_')) . ' COLLATE ' . $collation;
+    } else {
+        $create_query .= ' DEFAULT CHARACTER SET ' . $collation;
     }
     $create_query .= ';' . $crlf;
     if (!PMA_exportOutputHandler($create_query)) {
         return FALSE;
     }
-    if (isset($GLOBALS['sql_backquotes']) && PMA_MYSQL_INT_VERSION >= 40100 && isset($GLOBALS['sql_compatibility']) && $GLOBALS['sql_compatibility'] == 'NONE') {
-        return PMA_exportOutputHandler('USE ' . PMA_backquote($db) . ';' . $crlf);
+    if (isset($GLOBALS['sql_backquotes']) && isset($GLOBALS['sql_compatibility']) && $GLOBALS['sql_compatibility'] == 'NONE') {
+        $result = PMA_exportOutputHandler('USE ' . PMA_backquote($db) . ';' . $crlf);
+    } else {
+        $result = PMA_exportOutputHandler('USE ' . $db . ';' . $crlf);
     }
-    return PMA_exportOutputHandler('USE ' . $db . ';' . $crlf);
+
+    if ($result && isset($GLOBALS['sql_structure']) && isset($GLOBALS['sql_procedure_function'])) {
+        $text = '';
+        $delimiter = '$$';
+
+        $procedure_names = PMA_DBI_get_procedures_or_functions($db, 'PROCEDURE');
+        $function_names = PMA_DBI_get_procedures_or_functions($db, 'FUNCTION');
+
+        if ($procedure_names || $function_names) {
+            $text .= $crlf
+              . 'DELIMITER ' . $delimiter . $crlf;
+        }
+
+        if ($procedure_names) {
+            $text .=
+                PMA_exportComment()
+              . PMA_exportComment($GLOBALS['strProcedures'])
+              . PMA_exportComment();
+
+            foreach($procedure_names as $procedure_name) {
+                if (! empty($GLOBALS['sql_drop_table'])) {
+		    $text .= 'DROP PROCEDURE IF EXISTS ' . PMA_backquote($procedure_name) . $delimiter . $crlf;
+                }
+                $text .= PMA_DBI_get_definition($db, 'PROCEDURE', $procedure_name) . $delimiter . $crlf . $crlf;
+            }
+        }
+
+        if ($function_names) {
+            $text .=
+                PMA_exportComment()
+              . PMA_exportComment($GLOBALS['strFunctions'])
+              . PMA_exportComment();
+
+            foreach($function_names as $function_name) {
+                if (! empty($GLOBALS['sql_drop_table'])) {
+		    $text .= 'DROP FUNCTION IF EXISTS ' . PMA_backquote($function_name) . $delimiter . $crlf;
+                }
+                $text .= PMA_DBI_get_definition($db, 'FUNCTION', $function_name) . $delimiter . $crlf . $crlf;
+            }
+        }
+
+        if ($procedure_names || $function_names) {
+            $text .= 'DELIMITER ;' . $crlf;
+        }
+
+        if (! empty($text)) {
+            $result = PMA_exportOutputHandler($text);
+        }
+    }
+    return $result;
 }
 
 /**
@@ -298,7 +374,7 @@ function PMA_exportDBCreate($db)
  */
 function PMA_exportDBHeader($db)
 {
-    $head = PMA_exportComment() 
+    $head = PMA_exportComment()
           . PMA_exportComment($GLOBALS['strDatabase'] . ': ' . (isset($GLOBALS['sql_backquotes']) ? PMA_backquote($db) : '\'' . $db . '\''))
           . PMA_exportComment();
     return PMA_exportOutputHandler($head);
@@ -323,52 +399,37 @@ function PMA_exportDBFooter($db)
         unset($GLOBALS['sql_constraints']);
     }
 
-    if (PMA_MYSQL_INT_VERSION >= 50000 && isset($GLOBALS['sql_structure']) && isset($GLOBALS['sql_procedure_function'])) {
-        $procs_funcs = '';
+    if (isset($GLOBALS['sql_structure']) && isset($GLOBALS['sql_procedure_function'])) {
+        $text = '';
         $delimiter = '$$';
 
-        $procedure_names = PMA_DBI_get_procedures_or_functions($db, 'PROCEDURE');
-        $function_names = PMA_DBI_get_procedures_or_functions($db, 'FUNCTION');
+        if (PMA_MYSQL_INT_VERSION > 50100) {
+            $event_names = PMA_DBI_fetch_result('SELECT EVENT_NAME FROM information_schema.EVENTS WHERE EVENT_SCHEMA= \'' . PMA_sqlAddslashes($db,true) . '\';');
+        } else {
+            $event_names = array();
+        }
 
-        if ($procedure_names || $function_names) {
-            $procs_funcs .= $crlf
+        if ($event_names) {
+            $text .= $crlf
               . 'DELIMITER ' . $delimiter . $crlf;
-        }
 
-        if ($procedure_names) {
-            $procs_funcs .= 
-                PMA_exportComment() 
-              . PMA_exportComment($GLOBALS['strProcedures'])
-              . PMA_exportComment();
-
-            foreach($procedure_names as $procedure_name) {
-                if (! empty($GLOBALS['sql_drop_table'])) {
-		    $procs_funcs .= 'DROP PROCEDURE ' . PMA_backquote($procedure_name) . $delimiter . $crlf;
-                }	
-                $procs_funcs .= PMA_DBI_get_procedure_or_function_def($db, 'PROCEDURE', $procedure_name) . $delimiter . $crlf . $crlf;
-            }
-        }
-
-        if ($function_names) {
-            $procs_funcs .= 
+            $text .=
                 PMA_exportComment()
-              . PMA_exportComment($GLOBALS['strFunctions'])
+              . PMA_exportComment($GLOBALS['strEvents'])
               . PMA_exportComment();
 
-            foreach($function_names as $function_name) {
+            foreach($event_names as $event_name) {
                 if (! empty($GLOBALS['sql_drop_table'])) {
-		    $procs_funcs .= 'DROP FUNCTION ' . PMA_backquote($function_name) . $delimiter . $crlf;
-                }	
-                $procs_funcs .= PMA_DBI_get_procedure_or_function_def($db, 'FUNCTION', $function_name) . $delimiter . $crlf . $crlf;
+		    $text .= 'DROP EVENT ' . PMA_backquote($event_name) . $delimiter . $crlf;
+                }
+                $text .= PMA_DBI_get_definition($db, 'EVENT', $event_name) . $delimiter . $crlf . $crlf;
             }
+
+            $text .= 'DELIMITER ;' . $crlf;
         }
 
-        if ($procedure_names || $function_names) {
-            $procs_funcs .= 'DELIMITER ;' . $crlf;
-        }
-
-        if (!empty($procs_funcs)) {
-            $result = PMA_exportOutputHandler($procs_funcs);
+        if (! empty($text)) {
+            $result = PMA_exportOutputHandler($text);
         }
     }
     return $result;
@@ -387,13 +448,13 @@ function PMA_exportDBFooter($db)
  * @access  public
  */
 function PMA_getTableDefStandIn($db, $view, $crlf) {
-
     $create_query = '';
     if (! empty($GLOBALS['sql_drop_table'])) {
         $create_query .= 'DROP VIEW IF EXISTS ' . PMA_backquote($view) . ';' . $crlf;
     }
 
     $create_query .= 'CREATE TABLE ';
+
     if (isset($GLOBALS['sql_if_not_exists']) && $GLOBALS['sql_if_not_exists']) {
         $create_query .= 'IF NOT EXISTS ';
     }
@@ -415,6 +476,8 @@ function PMA_getTableDefStandIn($db, $view, $crlf) {
  * @param   string   the end of line sequence
  * @param   string   the url to go back in case of error
  * @param   boolean  whether to include creation/update/check dates
+ * @param   boolean  whether to add semicolon and end-of-line at the end
+ * @param   boolean  whether we're handling view
  *
  * @return  string   resulting schema
  *
@@ -424,13 +487,14 @@ function PMA_getTableDefStandIn($db, $view, $crlf) {
  *
  * @access  public
  */
-function PMA_getTableDef($db, $table, $crlf, $error_url, $show_dates = false)
+function PMA_getTableDef($db, $table, $crlf, $error_url, $show_dates = false, $add_semicolon = true, $view = false)
 {
     global $sql_drop_table;
     global $sql_backquotes;
     global $cfgRelation;
     global $sql_constraints;
     global $sql_constraints_query; // just the text of the query
+    global $sql_drop_foreign_keys;
 
     $schema_create = '';
     $auto_increment = '';
@@ -489,8 +553,14 @@ function PMA_getTableDef($db, $table, $crlf, $error_url, $show_dates = false)
     //
     // Note: SHOW CREATE TABLE, at least in MySQL 5.1.23, does not
     // produce a displayable result for the default value of a BIT
-    // field, nor does the mysqldump command. See MySQL bug 35796 
-    $result = PMA_DBI_query('SHOW CREATE TABLE ' . PMA_backquote($db) . '.' . PMA_backquote($table));
+    // field, nor does the mysqldump command. See MySQL bug 35796
+    $result = PMA_DBI_try_query('SHOW CREATE TABLE ' . PMA_backquote($db) . '.' . PMA_backquote($table));
+    // an error can happen, for example the table is crashed
+    $tmp_error = PMA_DBI_getError();
+    if ($tmp_error) {
+        return PMA_exportComment($GLOBALS['strInUse'] . '(' . $tmp_error . ')');
+    }
+
     if ($result != FALSE && ($row = PMA_DBI_fetch_row($result))) {
         $create_query = $row[1];
         unset($row);
@@ -504,6 +574,17 @@ function PMA_getTableDef($db, $table, $crlf, $error_url, $show_dates = false)
             $create_query = str_replace("\r", $crlf, $create_query);
         }
 
+        /*
+         * Drop database name from VIEW creation.
+         *
+         * This is a bit tricky, but we need to issue SHOW CREATE TABLE with
+         * database name, but we don't want name to show up in CREATE VIEW
+         * statement.
+         */
+        if ($view) {
+            $create_query = preg_replace('/' . PMA_backquote($db) . '\./', '', $create_query);
+        }
+
         // Should we use IF NOT EXISTS?
         if (isset($GLOBALS['sql_if_not_exists'])) {
             $create_query     = preg_replace('/^CREATE TABLE/', 'CREATE TABLE IF NOT EXISTS', $create_query);
@@ -535,7 +616,7 @@ function PMA_getTableDef($db, $table, $crlf, $error_url, $show_dates = false)
                         $sql_constraints = '';
                     } else {
                         $sql_constraints = $crlf
-                                         . PMA_exportComment() 
+                                         . PMA_exportComment()
                                          . PMA_exportComment($GLOBALS['strConstraintsForDumped'])
                                          . PMA_exportComment();
                     }
@@ -552,6 +633,7 @@ function PMA_getTableDef($db, $table, $crlf, $error_url, $show_dates = false)
                 // let's do the work
                 $sql_constraints_query .= 'ALTER TABLE ' . PMA_backquote($table) . $crlf;
                 $sql_constraints .= 'ALTER TABLE ' . PMA_backquote($table) . $crlf;
+                $sql_drop_foreign_keys .= 'ALTER TABLE ' . PMA_backquote($db) . '.' . PMA_backquote($table) . $crlf;
 
                 $first = TRUE;
                 for ($j = $i; $j < $sql_count; $j++) {
@@ -567,6 +649,11 @@ function PMA_getTableDef($db, $table, $crlf, $error_url, $show_dates = false)
                             $str_tmp = preg_replace('/(CONSTRAINT)/', 'ADD \1', $sql_lines[$j]);
                             $sql_constraints_query .= $str_tmp;
                             $sql_constraints .= $str_tmp;
+                            preg_match('/(CONSTRAINT)([\s])([\S]*)([\s])/', $sql_lines[$j], $matches);
+                            if (! $first) {
+                                $sql_drop_foreign_keys .= ', ';
+                            }
+                            $sql_drop_foreign_keys .= 'DROP FOREIGN KEY ' . $matches[3];
                         }
                         $first = FALSE;
                     } else {
@@ -590,7 +677,7 @@ function PMA_getTableDef($db, $table, $crlf, $error_url, $show_dates = false)
     $schema_create .= $auto_increment;
 
     PMA_DBI_free_result($result);
-    return $schema_create;
+    return $schema_create . ($add_semicolon ? ';' . $crlf : '');
 } // end of the 'PMA_getTableDef()' function
 
 
@@ -601,14 +688,13 @@ function PMA_getTableDef($db, $table, $crlf, $error_url, $show_dates = false)
  * @param   string   the table name
  * @param   string   the end of line sequence
  * @param   boolean  whether to include relation comments
- * @param   boolean  whether to include column comments
  * @param   boolean  whether to include mime comments
  *
  * @return  string   resulting comments
  *
  * @access  public
  */
-function PMA_getTableComments($db, $table, $crlf, $do_relation = false, $do_comments = false, $do_mime = false)
+function PMA_getTableComments($db, $table, $crlf, $do_relation = false,  $do_mime = false)
 {
     global $cfgRelation;
     global $sql_backquotes;
@@ -616,13 +702,6 @@ function PMA_getTableComments($db, $table, $crlf, $do_relation = false, $do_comm
 
     $schema_create = '';
 
-    // triggered only for MySQL < 4.1.x (pmadb-style comments)
-    if ($do_comments && $cfgRelation['commwork']) {
-        if (!($comments_map = PMA_getComments($db, $table))) {
-            unset($comments_map);
-        }
-    }
-
     // Check if we can use Relations (Mike Beck)
     if ($do_relation && !empty($cfgRelation['relation'])) {
         // Find which tables are related with the current one and write it in
@@ -644,39 +723,28 @@ function PMA_getTableComments($db, $table, $crlf, $do_relation = false, $do_comm
         }
     }
 
-    if (isset($comments_map) && count($comments_map) > 0) {
-        $schema_create .= $crlf
-                       . PMA_exportComment() 
-                       . PMA_exportComment($GLOBALS['strCommentsForTable']. ' ' . PMA_backquote($table, $sql_backquotes) . ':');
-        foreach ($comments_map AS $comment_field => $comment) {
-            $schema_create .= PMA_exportComment('  ' . PMA_backquote($comment_field, $sql_backquotes))
-                            . PMA_exportComment('      ' . PMA_backquote($comment, $sql_backquotes));
-        }
-        $schema_create .= PMA_exportComment();
-    }
-
     if (isset($mime_map) && count($mime_map) > 0) {
-        $schema_create .= $crlf
-                       . PMA_exportComment() 
+        $schema_create .= PMA_possibleCRLF() 
+                       . PMA_exportComment()
                        . PMA_exportComment($GLOBALS['strMIMETypesForTable']. ' ' . PMA_backquote($table, $sql_backquotes) . ':');
         @reset($mime_map);
         foreach ($mime_map AS $mime_field => $mime) {
             $schema_create .= PMA_exportComment('  ' . PMA_backquote($mime_field, $sql_backquotes))
                             . PMA_exportComment('      ' . PMA_backquote($mime['mimetype'], $sql_backquotes));
         }
-        $schema_create .= PMA_exportComment(); 
+        $schema_create .= PMA_exportComment();
     }
 
     if ($have_rel) {
-        $schema_create .= $crlf
-                       . PMA_exportComment() 
+        $schema_create .= PMA_possibleCRLF() 
+                       . PMA_exportComment()
                        . PMA_exportComment($GLOBALS['strRelationsForTable']. ' ' . PMA_backquote($table, $sql_backquotes) . ':');
         foreach ($res_rel AS $rel_field => $rel) {
             $schema_create .= PMA_exportComment('  ' . PMA_backquote($rel_field, $sql_backquotes))
                             . PMA_exportComment('      ' . PMA_backquote($rel['foreign_table'], $sql_backquotes)
                             . ' -> ' . PMA_backquote($rel['foreign_field'], $sql_backquotes));
         }
-        $schema_create .= PMA_exportComment(); 
+        $schema_create .= PMA_exportComment();
     }
 
     return $schema_create;
@@ -691,10 +759,14 @@ function PMA_getTableComments($db, $table, $crlf, $do_relation = false, $do_comm
  * @param   string   the end of line sequence
  * @param   string   the url to go back in case of error
  * @param   boolean  whether to include relation comments
- * @param   boolean  whether to include column comments
+ * @param   boolean  whether to include the pmadb-style column comments
+ *                   as comments in the structure; this is deprecated
+ *                   but the parameter is left here because export.php
+ *                   calls PMA_exportStructure() also for other export
+ *                   types which use this parameter
  * @param   boolean  whether to include mime comments
  * @param   string   'stand_in', 'create_table', 'create_view'
- * @param   string   'server', 'database', 'table' 
+ * @param   string   'server', 'database', 'table'
  *
  * @return  bool     Whether it suceeded
  *
@@ -705,22 +777,26 @@ function PMA_exportStructure($db, $table, $crlf, $error_url, $relation = FALSE,
     $formatted_table_name = (isset($GLOBALS['sql_backquotes']))
                           ? PMA_backquote($table)
                           : '\'' . $table . '\'';
-    $dump = $crlf
+    $dump = PMA_possibleCRLF() 
           . PMA_exportComment(str_repeat('-', 56))
-          . $crlf
-          . PMA_exportComment(); 
+          . PMA_possibleCRLF() 
+          . PMA_exportComment();
 
     switch($export_mode) {
         case 'create_table':
             $dump .=  PMA_exportComment($GLOBALS['strTableStructure'] . ' ' . $formatted_table_name)
-                  . PMA_exportComment(); 
-            $dump .= PMA_getTableDef($db, $table, $crlf, $error_url, $dates) . ';' . $crlf;
+                  . PMA_exportComment();
+            $dump .= PMA_getTableDef($db, $table, $crlf, $error_url, $dates);
+            $dump .= PMA_getTableComments($db, $table, $crlf, $relation, $mime);
+            break;
+        case 'triggers':
+            $dump = ''; 
             $triggers = PMA_DBI_get_triggers($db, $table);
             if ($triggers) {
-                $dump .=  $crlf
-                      . PMA_exportComment() 
+                $dump .=  PMA_possibleCRLF() 
+                      . PMA_exportComment()
                       . PMA_exportComment($GLOBALS['strTriggers'] . ' ' . $formatted_table_name)
-                      . PMA_exportComment(); 
+                      . PMA_exportComment();
                 $delimiter = '//';
                 foreach ($triggers as $trigger) {
                     $dump .= $trigger['drop'] . ';' . $crlf;
@@ -737,7 +813,7 @@ function PMA_exportStructure($db, $table, $crlf, $error_url, $relation = FALSE,
             if ($export_type != 'table') {
                 $dump .= 'DROP TABLE IF EXISTS ' . PMA_backquote($table) . ';' . $crlf;
             }
-            $dump .= PMA_getTableDef($db, $table, $crlf, $error_url, $dates) . ';' . $crlf;
+            $dump .= PMA_getTableDef($db, $table, $crlf, $error_url, $dates, true, true);
             break;
         case 'stand_in':
             $dump .=  PMA_exportComment($GLOBALS['strStandInStructureForView'] . ' ' . $formatted_table_name)
@@ -746,7 +822,6 @@ function PMA_exportStructure($db, $table, $crlf, $error_url, $relation = FALSE,
             $dump .= PMA_getTableDefStandIn($db, $table, $crlf);
     } // end switch
 
-    $dump .= PMA_getTableComments($db, $table, $crlf, $relation, $comments, $mime);
     // this one is built by PMA_getTableDef() to use in table copy/move
     // but not in the case of export
     unset($GLOBALS['sql_constraints_query']);
@@ -789,13 +864,13 @@ function PMA_exportData($db, $table, $crlf, $error_url, $sql_query)
 
     // Do not export data for a VIEW
     // (For a VIEW, this is called only when exporting a single VIEW)
-    if (PMA_Table::_isView($db, $table)) {
-        $head = $crlf
-          . PMA_exportComment() 
+    if (PMA_Table::isView($db, $table)) {
+        $head = PMA_possibleCRLF() 
+          . PMA_exportComment()
           . PMA_exportComment('VIEW ' . ' ' . $formatted_table_name)
-          . PMA_exportComment($GLOBALS['strData'] . ': ' . $GLOBALS['strNone']) 
-          . PMA_exportComment() 
-          . $crlf;
+          . PMA_exportComment($GLOBALS['strData'] . ': ' . $GLOBALS['strNone'])
+          . PMA_exportComment()
+          . PMA_possibleCRLF();
 
         if (! PMA_exportOutputHandler($head)) {
             return FALSE;
@@ -804,11 +879,10 @@ function PMA_exportData($db, $table, $crlf, $error_url, $sql_query)
     }
 
     // it's not a VIEW
-    $head = $crlf
-          . PMA_exportComment() 
+    $head = PMA_possibleCRLF() 
+          . PMA_exportComment()
           . PMA_exportComment($GLOBALS['strDumpingData'] . ' ' . $formatted_table_name)
-          . PMA_exportComment() 
-          . $crlf;
+          . PMA_exportComment();
 
     if (! PMA_exportOutputHandler($head)) {
         return FALSE;
@@ -821,8 +895,21 @@ function PMA_exportData($db, $table, $crlf, $error_url, $sql_query)
     //  are used, we did not get the true column name in case of aliases)
     $analyzed_sql = PMA_SQP_analyze(PMA_SQP_parse($sql_query));
 
-    $result      = PMA_DBI_query($sql_query, null, PMA_DBI_QUERY_UNBUFFERED);
+    $result      = PMA_DBI_try_query($sql_query, null, PMA_DBI_QUERY_UNBUFFERED);
+    // a possible error: the table has crashed
+    $tmp_error = PMA_DBI_getError();
+    if ($tmp_error) {
+        return PMA_exportOutputHandler(PMA_exportComment($GLOBALS['strInUse'] . ' (' . $tmp_error . ')'));
+    }
+
     if ($result != FALSE) {
+        // emit a single CRLF before the first data statement (produces
+        // an unintended CRLF when there is no data, but I don't see how it
+        // can be avoided, as we are in UNBUFFERED mode)
+        if (! PMA_exportOutputHandler($crlf)) {
+            return FALSE;
+        }
+
         $fields_cnt     = PMA_DBI_num_fields($result);
 
         // Get field information
@@ -941,7 +1028,9 @@ function PMA_exportData($db, $table, $crlf, $error_url, $sql_query)
                     $insert_line .= $field_set[$i] . ' = ' . $values[$i];
                 }
 
-                $insert_line .= ' WHERE ' . PMA_getUniqueCondition($result, $fields_cnt, $fields_meta, $row);
+                list($tmp_unique_condition, $tmp_clause_is_unique) = PMA_getUniqueCondition($result, $fields_cnt, $fields_meta, $row);
+                $insert_line .= ' WHERE ' . $tmp_unique_condition;
+                unset($tmp_unique_condition, $tmp_clause_is_unique);
 
             } else {
 
